<?php

namespace test\core\plugin;

/**
 * Test associated to \core\plugin\PluginManager
 */
class PluginManager extends \test\Test
{
	public function __construct()
	{
		$this->checks = array(
			$this->check_fetch_class,
			$this->check_fetch_plugins,
			$this->check_fetch_plugin,
		);
	}

	/**
	 * Check if fetch_class method work as expected
	 *
	 * @return \bool
	 */
	public function check_fetch_class() : bool
	{
		return (
			$this->check_fetch_class_bad_class_name('1') &
			$this->check_fetch_class_bad_class_name('\\') &
			$this->check_fetch_class_bad_class_name('') &
			$this->check_fetch_class_bad_class_name('\\core\\plugin\\Plugin Manager') &
			$this->check_fetch_class_good_class_name('€') &
			$this->check_fetch_class_good_class_name('\€\a') &
			$this->check_fetch_class_good_class_name('\\core\\plugin\\PluginManager')
		)
	}

	/**
	 * Check if fetch_plugins work as expected
	 *
	 * @return \bool
	 */
	public function check_fetch_plugins() : bool
	{
		\core\plugin\PluginManager::fetch_plugins(); # let's check if this throw an Exception at least
		return True; # NOTE: I don't see any possible test here
	}

	/**
	 * Checks if fetch_plugin work as expected
	 *
	 * @return \bool
	 */
	public function check_fetch_plugin() : bool
	{
		return (
			$this->check_fetch_plugin_incorrect_path('../class/core/plugin/') &&
			$this->check_fetch_plugin_plugin_state()
		)
	}

	/**
	 * Checks if the given path is incorrect
	 *
	 * @param \string
	 *
	 * @return \bool True if the the path is incorrect, False if it is
	 *               correct
	 */
	protected function check_fetch_plugin_incorrect_path(string $path) : bool
	{
		try
		{
			\core\plugin\PluginManager::fetch_plugin($path);
		}
		catch (\core\plugin\PluginException)
		{
			return True;
		}
		return False;
	}

	/**
	 * Checks if the plugin is incorrect as expected
	 *
	 * @return \bool
	 */
	protected function check_fetch_plugin_incorrect_plugin() : bool
	{
		\umask(0066);
		$created = False;
		if (!\is_dir('plugins'))
		{
			$created = True;
			\mkdir('plugins');
		}
		$plugin_name = \test\generate_random_plugin_name();
		$plugin_folder = \core\Path('plugins/' . $plugin_name);
		$template_folder = \core\Path('test/template/plugin/');
		\mkdir($plugin_folder);
		# Plugin folder is empty, should return an error
		try
		{
			\core\plugin\PluginManager::fetch_plugin($plugin_folder);
			\rmdir($plugin_folder);
			if ($created)
			{
				\rmdir('plugins');
			}
			return False;
		}
		catch (\core\plugin\PluginException)
		{
			continue;
		}

		\copy(
			from: $template_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
			to: $plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
		);
		# Plugin class is just an empty template
		try
		{
			\core\plugin\PluginManager::fetch_plugin($plugin_folder);
			\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
			\rmdir($plugin_folder);
			if ($created)
			{
				\rmdir('plugins');
			}
			return False;
		}
		catch (\core\plugin\PluginException)
		{
			continue;
		}

		/* Bad template */
		$template_folder->append('bad');
		\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
		foreach ($template_folder->walkDirs() as $folder)
		{
			\copy(
				from: $template_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
				to: $plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
			);
			\copy(
				from: $folder . \DIRECTORY_SEPARATOR . 'Config.class.php',
				to: $plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php',
			);
			\core\substitute_file(
				$plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
				array(
					'get_configuration' => \file_get_contents(
						$folder . \DIRECTORY_SEPARATOR . 'get_configuration.php'
					),
					'fetch_class' => \file_get_contents(
						$folder . \DIRECTORY_SEPARATOR . 'fetch_class.php'
					),
				),
			);
			try
			{
				\core\plugin\PluginManager::fetch_plugin($plugin_folder);
				\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php');
				\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
				\rmdir($plugin_folder);
				if ($created)
				{
					\rmdir('plugins');
				}
				return False;
			}
			catch (\core\plugin\PluginException)
			{
				continue;
			}
			\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php');
			\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
		}

		/* Good templates */
		$template_folder->parent()->append('good');
		foreach ($template_folder->walkDirs() as $folder)
		{
			\copy(
				from: $template_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
				to: $plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
			);
			\copy(
				from: $folder . \DIRECTORY_SEPARATOR . 'Config.class.php',
				to: $plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php',
			);
			\core\substitute_file(
				$plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php',
				array(
					'get_configuration' => \file_get_contents(
						$folder . \DIRECTORY_SEPARATOR . 'get_configuration.php'
					),
					'fetch_class' => \file_get_contents(
						$folder . \DIRECTORY_SEPARATOR . 'fetch_class.php'
					),
				),
			);
			try
			{
				\core\plugin\PluginManager::fetch_plugin($plugin_folder);
			}
			catch (\Exception)
			{
				\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php');
				\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
				\rmdir($plugin_folder);
				if ($created)
				{
					\rmdir('plugins');
				}
				return False;
			}
			\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Config.class.php');
			\unlink($plugin_folder . \DIRECTORY_SEPARATOR . 'Plugin.class.php');
		}

		\rmdir($plugin_folder);
		if ($created)
		{
			\rmdir('plugins');
		}
		return True;
	}

	/**
	 * Checks if the given class name is bad
	 *
	 * @param \string $class_name
	 *
	 * @return \bool True if bad class name, False if correct one
	 */
	protected function check_fetch_class_bad_class_name(string $class_name) : bool
	{
		try
		{
			\core\plugin\PluginManager::fetch_class('1'); # incorrect class name
		}
		catch (\core\plugin\PluginException)
		{
			return True
		}
		return False;
	}

	/**
	 * Checks if the given class name is good
	 *
	 * @param \string $class_name
	 *
	 * @return \bool True if good class name, False if bad one
	 */
	protected function check_fetch_class_good_class_name(string $class_name) : bool
	{
		return !$this->check_fetch_class_bad_class_name($class_name);
	}
}

?>
